@@ -1,18 +1,21 @@
 """OSSOS VOSpace storage convenience package"""
+import sys, traceback
+import math
 import errno
 import fnmatch
-import logging
 import os
 import re
 import tempfile
-import time
+import logging
 import warnings
 from glob import glob
+import time
 
 import requests as requests_module
 import vos
 from astropy import units
 from astropy.coordinates import SkyCoord
+from astropy.nddata import Cutout2D
 from astropy.io import ascii
 from astropy.io import fits
 from astropy.io.fits.verify import VerifyWarning
@@ -30,6 +33,8 @@ from .gui import config
 from .gui import logger
 from .wcs import WCS
 
+warnings.simplefilter('ignore', UserWarning)
+
 client = vos.Client()
 # from .gui.errorhandling import DownloadErrorHandler
 
@@ -295,7 +300,7 @@ def get_cands_uri(field, ccd, version='p', ext='measure3.cands.astrom', prefix=N
 
     measure3_dir = MEASURE3
     if block is not None:
-        measure3_dir + "/{}".format(block)
+        measure3_dir += "/{}".format(block)
     return "{}/{}{}{}{}{}".format(measure3_dir, prefix, field, version, ccd, ext)
 
 
@@ -630,12 +635,34 @@ def decompose_content_decomposition(content_decomposition):
     :return:
     """
     # check for '-*' in the cutout string and replace is naxis:1
-    content_decomposition = re.findall('(\d+)__(\d*)_(\d*)_(\d*)_(\d*)', content_decomposition)
+    content_decomposition = re.findall(r'(\d+)__(\d*)_(\d*)_(\d*)_(\d*)', content_decomposition)
     if len(content_decomposition) == 0:
         content_decomposition = [(0, 1, -1, 1, -1)]
     return content_decomposition
 
 
+def _reset_datasec(header, cutout):
+
+    default_datasec = "[{}:{},{}:{}]".format(1, header['NAXIS1'], 1, header['NAXIS2'])
+    datasec = header.get('DATASEC', default_datasec)
+    datasec = datasec_to_list(datasec)
+    corners = datasec
+    for idx in range(len(corners)):
+        try:
+            corners[idx] = int(cutout[idx+1])
+        except Exception:
+            pass
+
+    header['DATASEC'] = reset_datasec("[{}:{},{}:{}]".format(corners[0],
+                                                             corners[1],
+                                                             corners[2],
+                                                             corners[3]),
+                                       header.get('DATASEC', default_datasec),
+                                       header['NAXIS1'],
+                                       header['NAXIS2'])
+
+
+
 def _cutout_expnum(observation, sky_coord, radius):
     """
     Get a cutout from an exposure based on the RA/DEC location.
@@ -649,23 +676,59 @@ def _cutout_expnum(observation, sky_coord, radius):
     @rtype: list(HDUList)
     """
 
-    uri = observation.get_image_uri()
-    cutout_filehandle = tempfile.NamedTemporaryFile()
-    disposition_filename = client.copy(uri + "({},{},{})".format(sky_coord.ra.to('degree').value,
-                                                                 sky_coord.dec.to('degree').value,
-                                                                 radius.to('degree').value),
-                                       cutout_filehandle.name,
-                                       disposition=True)
-    cutouts = decompose_content_decomposition(disposition_filename)
+    filename = "{}.fits".format(observation.rawname)
+    if os.access(filename, os.R_OK):
+      try:
+        hdulist = fits.open(filename)
+        phdu = fits.PrimaryHDU()
+        phdu.header['ORIGIN'] = "OSSOS"
+        hdulist_out = fits.HDUList(phdu)
+        for hdu in hdulist:
+            if hdu.data is None:
+                continue
+            w = WCS(hdu.header)
+            x, y  = w.sky2xy(sky_coord.ra.degree, sky_coord.dec.degree)
+            radius = math.fabs(radius.to('degree').value / w.cd[0][0]) * 2.0
+            result = Cutout2D(hdu.data, (x, y), (radius, radius))
+            p1, p2 = result.to_cutout_position((hdu.header['CRPIX1'], hdu.header['CRPIX2']))
+            hdu = fits.ImageHDU(data=result.data, header=hdu.header)
+            hdu.header['CRPIX1'] = p1
+            hdu.header['CRPIX2'] = p2
+            hdu.header['XOFFSET'] = result.origin_cutout[0] - 1
+            hdu.header['YOFFSET'] = result.origin_cutout[1] - 1
+            hdu.converter = CoordinateConverter(hdu.header['XOFFSET'], hdu.header['YOFFSET'])
+            hdu.wcs = WCS(hdu.header)
+            bbox = result.bbox_original
+            # package the Cutout2D Bounding box to match cutout format from CADC
+            cutout = [0,bbox[1][0], bbox[1][1], bbox[0][0], bbox[0][1]]
+            _reset_datasec(hdu.header, cutout)
+            # hdu.header.update(result.wcs.to_header())
+            hdulist_out.append(hdu)
+        return hdulist_out
+      except Exception as ex:
+        exc_type, exc_value, exc_traceback = sys.exc_info()
+        traceback.print_tb(exc_traceback, limit=100, file=sys.stdout)
+        print(ex)
+        raise(ex)
 
-    cutout_filehandle.seek(0)
-    with warnings.catch_warnings():
-        warnings.simplefilter('ignore', FITSFixedWarning)
-        hdulist = fits.open(cutout_filehandle, mode='update', lazy_load_hdus=False,
-                            memmap=False)
 
-        hdulist.verify('silentfix+ignore')
-    logger.debug("Initial Length of HDUList: {}".format(len(hdulist)))
+    else:
+      uri = observation.get_image_uri()
+      cutout_filehandle = tempfile.NamedTemporaryFile()
+      disposition_filename = client.copy(uri + "({:f},{:f},{:f})".format(sky_coord.ra.to('degree').value,
+                                                                   sky_coord.dec.to('degree').value,
+                                                                   radius.to('degree').value),
+                                         cutout_filehandle.name,
+                                         disposition=True)
+      cutouts = decompose_content_decomposition(disposition_filename)
+
+      cutout_filehandle.seek(0)
+      with warnings.catch_warnings():
+          warnings.simplefilter('ignore', FITSFixedWarning)
+          hdulist = fits.open(cutout_filehandle, mode='update', lazy_load_hdus=False,
+                              memmap=False)
+          hdulist.verify('silentfix+ignore')
+      logger.debug("Initial Length of HDUList: {}".format(len(hdulist)))
 
     # Make sure here is a primaryHDU
     if len(hdulist) == 1:
@@ -696,7 +759,7 @@ def _cutout_expnum(observation, sky_coord, radius):
             except Exception:
                 pass
 
-        hdu.header['DATASEC'] = reset_datasec("[{}:{},{}:{}]".format(corners[0],
+        hdu.header['DATASEC'] = reset_datasec("[{:d}:{:d},{:d}:{:d}]".format(corners[0],
                                                                      corners[1],
                                                                      corners[2],
                                                                      corners[3]),
@@ -735,7 +798,7 @@ def ra_dec_cutout(uri, sky_coord, radius, update_wcs=False):
         import http.client as http_client
     except ImportError:
         # Python 2
-        import http.client as http_client
+        import httplib as http_client
 
     # Get the 'uncut' images CRPIX1/CRPIX2 values
 
@@ -819,18 +882,27 @@ def ra_dec_cutout(uri, sky_coord, radius, update_wcs=False):
     logger.debug("Sending back {}".format(hdulist))
     return hdulist
 
+
 def frame2expnum(frameid):
-    """Given a standard OSSOS frameid return the expnum, version and ccdnum as a dictionary."""
-    result = {}
-    parts = re.search('(?P<expnum>\d{7})(?P<type>\S)(?P<ccd>\d\d)', frameid)
-    assert parts is not None
-    result['expnum'] = parts.group('expnum')
-    result['ccd'] = parts.group('ccd')
-    result['version'] = parts.group('type')
-    return result
+    """Given a standard OSSOS frameid return the expnum, version and ccdnum
+    as a dictionary."""
+    try:
+        f = re.search("(?P<prefix>\D*)(?P<expnum>\d*)(?P<version>\D*)(?P<ccd>\d*)",
+                      frameid)
+        return f.groupdict()
+    except Exception as ex:
+        logging.error(f"Failed to parse: {frameid}")
+        logging.error(str(ex))
+        raise ex
+
 
 def get_frame(frameid, cutout=None):
-    """Given a frameid, which consists of expnum version and ccd retrieve a cutout."""
+    """retrieve cutout for given frameid
+    @param frameid: OSSOS Frame identifier
+    @ptype frameid: str
+    @param cutout: cfitsio cutout.
+    @ptype cutout: str
+    """
     return get_image(cutout=cutout, **frame2expnum(frameid))
 
 
@@ -1167,7 +1239,7 @@ def get_fwhm(expnum, ccd, prefix=None, version='p'):
         fwhm[uri] = float(open_vos_or_local(uri).read())
         return fwhm[uri]
     except Exception as ex:
-        logger.error(str(ex))
+        logger.warning(str(ex))
         fwhm[uri] = 5.0
         return fwhm[uri]
 
@@ -1286,8 +1358,16 @@ def copy(source, dest):
     @return:
     """
     logger.info("copying {} -> {}".format(source, dest))
-
-    return client.copy(source, dest)
+    n = 1
+    ex = IOError("no error")
+    while n < 10:
+        try:
+            return client.copy(source, dest)
+        except Exception as ex:
+            logger.debug(f"{source} -> {dest} failed with {ex}.  Retrying")
+            time.sleep(2)
+            n += 1
+    raise ex
 
 
 def vlink(s_expnum, s_ccd, s_version, s_ext,
@@ -1430,16 +1510,14 @@ def set_property(node_uri, property_name, property_value, ossos_base=True):
     @param ossos_base:
     @return:
     """
+    property_uri = tag_uri(property_name) if ossos_base else property_name
     while True:
         try:
-            logger.info(f"Getting Node Property")
             node = client.get_node(node_uri)
-            logger.info(f"{node}")
-            property_uri = tag_uri(property_name) if ossos_base else property_name
-            logger.info(f"{property_uri}")
             break
-        except exceptions.HttpException:
-            print(f'Waiting to retry VOSpace')
+        except exceptions.HttpException as ex:
+            raise ex
+            logger.warning(f"While getting node property {property_uri}: {ex}")
             time.sleep(3)
 
     # If there is an existing value, clear it first
@@ -1447,21 +1525,21 @@ def set_property(node_uri, property_name, property_value, ossos_base=True):
         while True:
             try:
                     node.props[property_uri] = None
-                    logger.info(f"Clearing Node Property")
                     client.add_props(node)
                     break
-            except exceptions.HttpException:
-                print(f'Waiting to retry VOSpace')
+            except exceptions.HttpException as ex:
+                raise ex
+                logger.warning(f"While clearing node property {property_uri}: {ex}")
                 time.sleep(3)
 
     while True:
         try:
             node.props[property_uri] = property_value
-            logger.info(f"Adding Node Property {property_uri}: {property_value}")
             client.add_props(node)
             break
-        except exceptions.HttpException:
-            print(f'Waiting to retry VOSpace')
+        except exceptions.HttpException as ex:
+            raise ex
+            logger.warning(f"While setting node property {property_uri}: {ex}")
             time.sleep(3)
 
 
@@ -1625,7 +1703,6 @@ def get_header(uri):
 def get_astheader(expnum, ccd, version='p', prefix=None):
     """
     Retrieve the header for a given dbimages file.
-
     @param expnum:  CFHT odometer number
     @param ccd: which ccd based extension (0..35)
     @param version: 'o','p', or 's'
